package uk.ac.derby.lpt.sili2.interpreter;

import uk.ac.derby.lpt.sili2.machine.Representation;
import uk.ac.derby.lpt.sili2.machine.RepresentationFloat;
import uk.ac.derby.lpt.sili2.machine.RepresentationInteger;
import uk.ac.derby.lpt.sili2.machine.RepresentationString;
import uk.ac.derby.lpt.sili2.types.*;

/** A value. */
public class Value {
	
	private Representation representation;
	private Type type;

	public static Value getStringValueFromDelimitedCharacters(String s) {
		return new Value(TypeString.getInstance(), RepresentationString.stripDelimited(s));
	}

	public static Value getIntegerValueFromCharacters(String s) {
		return new Value(TypeInteger.getInstance(), new RepresentationInteger(Long.parseLong(s)));
	}

	public static Value getFloatValueFromCharacters(String s) {
		return new Value(TypeFloat.getInstance(), new RepresentationFloat(Double.parseDouble(s)));
	}
	
	public Value(Type type, Representation representation) {
		this.type = type;
		this.representation = representation;
	}
	
	/** Get this value's type. */
	public Type getType() {
		return type;
	}

	/** Get this value's representation. */
	public Representation getRepresentation() {
		return representation;
	}
	
	/** Perform logical OR on this value and another. */
	public Value or(Value v) {
		return type.or(this, v);
	}
	
	/** Perform logical AND on this value and another. */
	public Value and(Value v) {
		return type.and(this, v);
	}
	
	/** Perform logical NOT on this value. */  
	public Value not() {
		return type.not(this);
	}
	
	/** Compare this value and another. */
	public int compare(Value v) {
		return type.compare(this, v);
	}
	
	/** Add this value to another. */
	public Value add(Value v) {
		return type.add(this, v);
	}
	
	/** Subtract another value from this. */
	public Value subtract(Value v) {
		return type.subtract(this, v);
	}
	
	/** Multiply this value with another. */
	public Value mult(Value v) {
		return type.mult(this, v);
	}
	
	/** Divide another value by this. */
	public Value div(Value v) {
		return type.div(this, v);
	}
	
	/** Return unary plus of this value. */
	public Value unary_plus() {
		return type.unary_plus(this);
	}
	
	/** Return unary minus of this value. */
	public Value unary_minus() {
		return type.unary_minus(this);
	}
	
	/** Test this value and another for equality. */
	public Value eq(Value v) {
		return type.eq(this, v);
	}
	
	/** Test this value and another for non-equality. */
	public Value neq(Value v) {
		return type.neq(this, v);
	}
	
	/** Test this value and another for >= */
	public Value gte(Value v) {
		return type.gte(this, v);
	}
	
	/** Test this value and another for <= */
	public Value lte(Value v) {
		return type.lte(this, v);
	}
	
	/** Test this value and another for > */
	public Value gt(Value v) {
		return type.gt(this, v);
	}
	
	/** Test this value and another for < */	
	public Value lt(Value v) {
		return type.lt(this, v);
	}

	/** Return true if this is true. */
	public boolean isTrue() {
		return type.isTrue(this);
	}
	
	public void write() {
		type.write(this);
	}
}
