package uk.ac.derby.lpt.sili2.types;

import uk.ac.derby.lpt.sili2.interpreter.ExceptionSemantic;
import uk.ac.derby.lpt.sili2.interpreter.Value;

public abstract class TypeAbstract implements Type {

	public abstract String getName();
		
	public int compare(Value v1, Value v2) {
		return v1.getRepresentation().compare(v2.getRepresentation());
	}
	
	public Value or(Value v1, Value v2) {
		throw new ExceptionSemantic("Cannot perform OR on " + v1.getType().getName() + " and " + v2.getType().getName());
	}

	public Value and(Value v1, Value v2) {
		throw new ExceptionSemantic("Cannot perform AND on " + v1.getType().getName() + " and " + v2.getType().getName());
	}

	public Value not(Value v) {
		throw new ExceptionSemantic("Cannot perform NOT on " + v.getType().getName());
	}

	public Value add(Value v1, Value v2) {
		throw new ExceptionSemantic("Cannot perform + on " + v1.getType().getName() + " and " + v2.getType().getName());
	}

	public Value subtract(Value v1, Value v2) {
		throw new ExceptionSemantic("Cannot perform - on " + v1.getType().getName() + " and " + v2.getType().getName());
	}

	public Value mult(Value v1, Value v2) {
		throw new ExceptionSemantic("Cannot perform * on " + v1.getType().getName() + " and " + v2.getType().getName());
	}

	public Value div(Value v1, Value v2) {
		throw new ExceptionSemantic("Cannot perform / on " + v1.getType().getName() + " and " + v2.getType().getName());
	}

	public Value unary_plus(Value v) {
		throw new ExceptionSemantic("Cannot perform + on " + v.getType().getName());
	}

	public Value unary_minus(Value v) {
		throw new ExceptionSemantic("Cannot perform - on " + v.getType().getName());
	}

	/** Test this value and another for equality. */
	public Value eq(Value v1, Value v2) {
		return TypeBoolean.getBoolean(compare(v1, v2) == 0);
	}
	
	/** Test this value and another for non-equality. */
	public Value neq(Value v1, Value v2) {
		return TypeBoolean.getBoolean(compare(v1, v2) != 0);
	}
	
	/** Test this value and another for >= */
	public Value gte(Value v1, Value v2) {
		return TypeBoolean.getBoolean(compare(v1, v2) >= 0);		
	}
	
	/** Test this value and another for <= */
	public Value lte(Value v1, Value v2) {
		return TypeBoolean.getBoolean(compare(v1, v2) <= 0);
	}
	
	/** Test this value and another for > */
	public Value gt(Value v1, Value v2) {
		return TypeBoolean.getBoolean(compare(v1, v2) > 0);
	}
	
	/** Test this value and another for < */	
	public Value lt(Value v1, Value v2) {
		return TypeBoolean.getBoolean(compare(v1, v2) < 0);
	}

	public boolean isTrue(Value value) {
		return value.getRepresentation().booleanValue();
	}

	public void write(Value value) {
		value.getRepresentation().write();
	}

}
